﻿B4A=true
Group=Default Group
ModulesStructureVersion=1
Type=StaticCode
Version=8
@EndOfDesignText@
'ExpandView - Code module
' V. 2.0

Sub Process_Globals
	Public Const EXPVW_DIR_LEFT_TO_RIGHT As Int = 1
	Public Const EXPVW_DIR_RIGHT_TO_LEFT As Int = 2
	Public Const EXPVW_DIR_TOP_TO_BOTTOM As Int = 3
	Public Const EXPVW_DIR_BOTTOM_TO_TOP As Int = 4
	
	Public Const EXPVW_ANIM_INVERT As Int = 0
	Public Const EXPVW_ANIM_COLLAPSE As Int = 1
	Public Const EXPVW_ANIM_EXPAND As Int = 2

	Private mmapViews As Map
	
	Private Const ANIM_DURATION As Int = 400
	
	Type tExpVwItem(Vw As Object, Left As Int, Top As Int, Width As Int, Height As Int, BorderSize As Int, CollapsedMinSize As Int, Duration As Int, Direction As Int, IsCollapsed As Boolean)
	
End Sub

Public Sub Init
	If Not(mmapViews.IsInitialized) Then
		mmapViews.Initialize
	Else
		mmapViews.Clear
	End If
End Sub

' [objB4XView] please, pay attention: you must pass B4XView or a "native" view, of course.
' [Duration] pass -1 to use the default, 400 ms.
' [Direction] pass one of the constants provided, which start with EXPVW_DIR_.
'      pass -1 to use the default, EXPVW_DIR_LEFT_TO_RIGHT.
' [PanelBorderSize] if the view you want to animate is a panel you must pass its border size;
'      pass 0 otherwise.
' [CollapsedMinSize] usually zero but you may want to set a minimum of size to be able to touch
'      and expand the view. It is, obviously, related to the Direction.
Public Sub AddView(objB4XView As Object, PanelBorderSize As Int, CollapsedMinSize As Int, Duration As Int, Direction As Int)
	Dim Vw As B4XView = objB4XView

	Dim ExpVwItem As tExpVwItem
	ExpVwItem.Initialize
	ExpVwItem.Vw = objB4XView
	ExpVwItem.Left = Vw.Left
	ExpVwItem.Top = Vw.Top
	ExpVwItem.Width = Vw.Width
	ExpVwItem.Height = Vw.Height
	PanelBorderSize = Max(0, PanelBorderSize)
	ExpVwItem.BorderSize = PanelBorderSize
	CollapsedMinSize = Max(0, CollapsedMinSize)
	ExpVwItem.CollapsedMinSize = CollapsedMinSize
	If Not(Duration = - 1) Then
		ExpVwItem.Duration = Duration
	Else
		ExpVwItem.Duration = ANIM_DURATION
	End If
	If Not(Direction = - 1) Then
		ExpVwItem.Direction = Direction
	Else
		ExpVwItem.Direction = EXPVW_DIR_LEFT_TO_RIGHT
	End If
	ExpVwItem.IsCollapsed = False

	mmapViews.Put(objB4XView, ExpVwItem)
End Sub

' Expands or collapses the view (depending on the current state).
'[objB4XView] please, pay attention: you must pass B4XView or a "native" view, of course.
'   It must be previously added using AddView.
' [Duration] pass -1 to use the default (400) or the last set.
Public Sub Animate(objB4XView As Object, Duration As Int)
	Dim Vw As B4XView = objB4XView
	Dim FinalLeft, FinalTop, FinalWidth, FinalHeight As Int
	Dim Gap As Int ' due to a bug in SetLayoutAnimated.

	Dim ExpVwItem As tExpVwItem = mmapViews.Get(objB4XView)
	ExpVwItem = mmapViews.Get(objB4XView)
	Gap = 1dip + ExpVwItem.BorderSize
	
	If Duration <> - 1 Then
		ExpVwItem.Duration = Duration
	End If

	FinalLeft = Vw.Left
	FinalTop = Vw.Top
	FinalWidth = Vw.Width
	FinalHeight = Vw.Height
	
	If ExpVwItem.isCollapsed Then
		' Expand
		Select ExpVwItem.Direction
			Case EXPVW_DIR_BOTTOM_TO_TOP
				FinalHeight = ExpVwItem.Height
				FinalTop = ExpVwItem.Top
			Case EXPVW_DIR_TOP_TO_BOTTOM
				FinalHeight = ExpVwItem.Height
			Case EXPVW_DIR_LEFT_TO_RIGHT
				FinalWidth = ExpVwItem.Width
			Case EXPVW_DIR_RIGHT_TO_LEFT
				FinalWidth = ExpVwItem.Width
				FinalLeft = ExpVwItem.Left
		End Select
	Else
		' Collapse.
		Select ExpVwItem.Direction
			Case EXPVW_DIR_BOTTOM_TO_TOP
				FinalHeight = ExpVwItem.CollapsedMinSize
				FinalTop = ExpVwItem.Top + ExpVwItem.Height - ExpVwItem.CollapsedMinSize
			Case EXPVW_DIR_TOP_TO_BOTTOM
				FinalHeight = ExpVwItem.CollapsedMinSize
			Case EXPVW_DIR_LEFT_TO_RIGHT
				FinalWidth = ExpVwItem.CollapsedMinSize
			Case EXPVW_DIR_RIGHT_TO_LEFT
				FinalLeft = ExpVwItem.Left + ExpVwItem.Width - ExpVwItem.CollapsedMinSize
				FinalWidth = ExpVwItem.CollapsedMinSize
		End Select
	End If
	
	Vw.SetLayoutAnimated(ExpVwItem.Duration, FinalLeft, FinalTop, FinalWidth + Gap, FinalHeight + Gap)
	Sleep(ExpVwItem.Duration + 20)
	Vw.SetLayoutAnimated(10, FinalLeft, FinalTop, FinalWidth, FinalHeight)
	ExpVwItem.isCollapsed = Not(ExpVwItem.isCollapsed)
End Sub
